@@ -1,18 +1,18 @@
-"""
-Representation of items in a Doorstop document.
-"""
+"""Representation of items in a Doorstop document."""
 
 import os
 import re
 import logging
 
-from doorstop.core.base import auto_load, auto_save, BaseFileObject, Literal
+from doorstop.core import base
+from doorstop.core.base import auto_load, auto_save, BaseFileObject
 from doorstop import common
 from doorstop.common import DoorstopError, DoorstopWarning, DoorstopInfo
 from doorstop import settings
 
 
 class Item(BaseFileObject):  # pylint: disable=R0904
+
     """Represents an item file with linkable text."""
 
     EXTENSIONS = '.yml', '.yaml'
@@ -27,11 +27,9 @@ class Item(BaseFileObject):  # pylint: disable=R0904
     def __init__(self, path, root=os.getcwd()):
         """Load an item from an existing file.
 
-        Internally, this constructor is also used to initialize new
-        items by providing default properties.
-
         @param path: path to Item file
         @param root: path to root of project
+
         """
         super().__init__()
         # Ensure the path is valid
@@ -80,7 +78,7 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         return self.level < other.level
 
     @staticmethod
-    def new(path, root, prefix, sep, digits, number, level, auto=None):  # pylint: disable=R0913
+    def new(path, root, prefix, sep, digits, number, level=None, auto=None):  # pylint: disable=R0913
         """Create a new item.
 
         @param path: path to directory for the new item
@@ -89,10 +87,11 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         @param sep: separator between prefix and number
         @param digits: number of digits for the new document
         @param number: number for the new item
-        @param level: level for the new item (None for default)
+        @param level: level for the new item
         @param auto: enables automatic save
 
         @raise DoorstopError: if the item already exists
+
         """
         identifier = join_id(prefix, sep, number, digits)
         filename = identifier + Item.EXTENSIONS[0]
@@ -103,7 +102,7 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         # Initialize the item
         item = Item(path2, root=root)
         item.auto = False
-        item.level = level or Item.DEFAULT_LEVEL
+        item.level = level if level is not None else item.level
         if auto or (auto is None and Item.auto):
             item.save()
         # Return the item
@@ -121,7 +120,7 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         # Store parsed data
         for key, value in data.items():
             if key == 'level':
-                self._data['level'] = convert_level(value)
+                self._data['level'] = load_level(value)
             elif key == 'active':
                 self._data['active'] = bool(value)
             elif key == 'normative':
@@ -129,12 +128,14 @@ class Item(BaseFileObject):  # pylint: disable=R0904
             elif key == 'derived':
                 self._data['derived'] = bool(value)
             elif key == 'text':
-                self._data['text'] = value.strip()
+                self._data['text'] = load_text(value)
             elif key == 'ref':
                 self._data['ref'] = value.strip()
             elif key == 'links':
                 self._data['links'] = set(value)
             else:
+                if isinstance(value, str):
+                    value = load_text(value)
                 self._data[key] = value
         # Set meta attributes
         self._loaded = True
@@ -146,20 +147,20 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         data = {}
         for key, value in self._data.items():
             if key == 'level':
-                level = '.'.join(str(n) for n in value)
-                if len(value) == 1:
-                    level = int(level)
-                elif len(value) == 2:
-                    level = float(level)
-                data['level'] = level
+                data['level'] = save_level(value)
             elif key == 'text':
-                data['text'] = Literal(sbd(self._data['text']))
+                data['text'] = save_text(self._data['text'])
             elif key == 'ref':
                 data['ref'] = value.strip()
             elif key == 'links':
                 data['links'] = sorted(value)
             else:
-                # TODO: dump long strings as Literal (and SBD?)
+                if isinstance(value, str):
+                    # length of "key_text: value_text"
+                    lenth = len(key) + 2 + len(value)
+                    if lenth > settings.MAX_LINE_LENTH or '\n' in value:
+                        end = '\n' if value.endswith('\n') else ''
+                        value = save_text(value, end=end)
                 data[key] = value
         # Dump the data to YAML
         text = self._dump(data)
@@ -208,7 +209,7 @@ class Item(BaseFileObject):  # pylint: disable=R0904
     @auto_save
     def level(self, value):
         """Set the item's level."""
-        self._data['level'] = convert_level(value)
+        self._data['level'] = load_level(value)
 
     @property
     def depth(self):
@@ -247,6 +248,7 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         A derived item does not have links to items in its parent
         document, but should still be linked to by items in its child
         documents.
+
         """
         return self._data['derived']
 
@@ -278,9 +280,10 @@ class Item(BaseFileObject):  # pylint: disable=R0904
 
     @property
     def heading(self):
-        """Indicates if the item is a heading.
+        """Indicate if the item is a heading.
 
         Headings have a level that ends in zero and are non-normative.
+
         """
         return self.level[-1] == 0 and not self.normative
 
@@ -306,7 +309,7 @@ class Item(BaseFileObject):  # pylint: disable=R0904
     @auto_save
     def text(self, value):
         """Set the item's text."""
-        self._data['text'] = value
+        self._data['text'] = str(value) if value else ""
 
     @property
     @auto_load
@@ -315,6 +318,7 @@ class Item(BaseFileObject):  # pylint: disable=R0904
 
         An external reference can be part of a line in a text file or
         the filename of any type of file.
+
         """
         return self._data['ref']
 
@@ -322,7 +326,7 @@ class Item(BaseFileObject):  # pylint: disable=R0904
     @auto_save
     def ref(self, value):
         """Set the item's external file reference."""
-        self._data['ref'] = value
+        self._data['ref'] = str(value) if value else ""
 
     @property
     @auto_load
@@ -360,8 +364,9 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         @param tree: Tree containing the item
 
         @return: indication that the item is valid
+
         """
-        # TODO: this could be common code with Item/Document/Tree
+        # TODO: refactor: this could be common code with Item/Document/Tree
         valid = True
         # Display all issues
         for issue in self.issues(document=document, tree=tree):
@@ -383,6 +388,7 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         @param tree: Tree containing the item
 
         @return: generator of DoorstopError, DoorstopWarning, DoorstopInfo
+
         """
         logging.info("checking item {}...".format(self))
         # Verify the file can be parsed
@@ -391,15 +397,18 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         if not self.active:
             logging.info("skipped inactive item: {}".format(self))
             return
-        self.auto = False
+        # Delay item save if reformatting
+        if settings.REFORMAT:
+            self.auto = False
         # Check text
         if not self.text and not self.ref:
             yield DoorstopWarning("no text")
         # Check external references
-        try:
-            self.find_ref(ignored=tree.vcs.ignored if tree else None)
-        except DoorstopError as exc:
-            yield exc
+        if settings.CHECK_REF:
+            try:
+                self.find_ref(ignored=tree.vcs.ignored if tree else None)
+            except DoorstopError as exc:
+                yield exc
         # Check links
         if not self.normative and self.links:
             yield DoorstopWarning("non-normative, but has links")
@@ -413,7 +422,8 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         if document and tree:
             yield from self._issues_both(document, tree)
         # Reformat the file
-        self.save()
+        if settings.REFORMAT:
+            self.save()
 
     def _issues_document(self, document):
         """Yield all the item's issues against its document."""
@@ -464,12 +474,13 @@ class Item(BaseFileObject):  # pylint: disable=R0904
                 logging.debug("found linked item: {}".format(identifier))
                 identifiers.add(identifier)
         # Apply the reformatted item IDs
-        self._data['links'] = identifiers
+        if settings.REFORMAT:
+            self._data['links'] = identifiers
 
     def _issues_both(self, document, tree):
         """Yield all the item's issues against its document and tree."""
         # Verify an item is being linked to (reverse links)
-        if self.normative:
+        if settings.CHECK_RLINKS and self.normative:
             rlinks, children = self.find_rlinks(document, tree, find_all=False)
             if not rlinks:
                 for child in children:
@@ -487,6 +498,7 @@ class Item(BaseFileObject):  # pylint: disable=R0904
                  None, None (when no ref)
 
         @raise DoorstopError: when no ref is found
+
         """
         if not self.ref:
             logging.debug("no external reference to search for")
@@ -508,8 +520,6 @@ class Item(BaseFileObject):  # pylint: disable=R0904
                     continue
                 # Skip ignored paths
                 if ignored(path):
-                    # TODO: remove this logging line
-                    logging.debug("skipped: {}".format(path))
                     continue
                 # Search for the reference in the file
                 if filename == self.ref:
@@ -534,19 +544,20 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         @param find_all: find all items (not just the first) before returning
 
         @return: list of found item IDs, list of all child Documents
+
         """
         rlinks = []
         children = []
         for document2 in tree:
             if document2.parent == document.prefix:
                 children.append(document2)
-            # Search for reverse links unless we only need to find one
-            if not rlinks or find_all:
-                for item in document2:
-                    if self.id in item.links:
-                        rlinks.append(item.id)
-                        if not find_all:
-                            break
+                # Search for reverse links unless we only need to find one
+                if not rlinks or find_all:
+                    for item in document2:
+                        if self.id in item.links:
+                            rlinks.append(item.id)
+                            if not find_all:
+                                break
         if rlinks:
             if find_all:
                 msg = "reverse links: {}".format(', '.join(rlinks))
@@ -556,6 +567,7 @@ class Item(BaseFileObject):  # pylint: disable=R0904
         return rlinks, children
 
     def delete(self, path=None):
+        """Delete the item."""
         super().delete(self.path)
 
 
@@ -595,59 +607,84 @@ def join_id(prefix, sep, number, digits):
     return "{}{}{}".format(prefix, sep, str(number).zfill(digits))
 
 
-def convert_level(text):
-    """Convert a level string to a tuple.
+def load_text(value):
+    r"""Convert dumped text to the original string.
 
-    >>> convert_level("1.2.3")
+    >>> load_text("abc\ndef")
+    'abc def'
+
+    >>> load_text("list:\n\n- a\n- b\n")
+    'list:\n\n- a\n- b'
+
+    """
+    return base.join(value)
+
+
+def save_text(text, end='\n'):
+    """Break a string at sentences and dump as literal YAML with wrapping."""
+    return base.Literal(base.wrap(base.sbd(text, end=end)))
+
+
+def load_level(value):
+    """Convert an iterable, number, or level string to a tuple.
+
+    >>> load_level("1.2.3")
     (1, 2, 3)
 
-    >>> convert_level(['4', '5'])
+    >>> load_level(['4', '5'])
     (4, 5)
 
-    >>> convert_level(4.2)
+    >>> load_level(4.2)
     (4, 2)
 
-    >>> convert_level([7, 0, 0])
+    >>> load_level([7, 0, 0])
     (7, 0)
 
-    >>> convert_level(1)
+    >>> load_level(1)
     (1,)
 
     """
-    # Correct for integers (42) and floats (4.2) in YAML
-    if isinstance(text, int) or isinstance(text, float):
-        text = str(text)
+    # Correct for integers (e.g. 42) and floats (e.g. 4.2) in YAML
+    if isinstance(value, (int, float)):
+        value = str(value)
+
     # Split strings by periods
-    if isinstance(text, str):
-        nums = text.split('.')
-    else:
-        nums = text
+    if isinstance(value, str):
+        nums = value.split('.')
+    else:  # assume an iterable
+        nums = value
+
     # Clean up multiple trailing zeros
     parts = [int(n) for n in nums]
     if parts[-1] == 0:
         while parts[-1] == 0:
             del parts[-1]
         parts.append(0)
+
     # Convert the level to a tuple
     return tuple(parts)
 
 
-# http://en.wikipedia.org/wiki/Sentence_boundary_disambiguation
-SBD = re.compile(r"((?<=[a-z0-9][.?!])|(?<=[a-z0-9][.?!]\"))(\s|\r\n)(?=\"?[A-Z])")  # pylint: disable=C0301
+def save_level(parts):
+    """Convert a level's part into non-quoted YAML value.
 
+    >>> save_level((1,))
+    1
 
-def sbd(text):
-    """Replace sentence boundaries with newlines and append a newline.
+    >>> save_level((1,0))
+    1.0
 
-    >>> sbd("Hello, world!")
-    'Hello, world!\\n'
-
-    >>> sbd("Hello, world! How are you? I'm fine. Good.")
-    "Hello, world!\\nHow are you?\\nI'm fine.\\nGood.\\n"
+    >>> save_level((1,0,0))
+    '1.0.0'
 
     """
-    stripped = text.strip()
-    if stripped:
-        return SBD.sub('\n', stripped) + '\n'
-    else:
-        return ''
+    # Join the level's parts
+    level = '.'.join(str(n) for n in parts)
+
+    # Convert formats to cleaner YAML formats
+    if len(parts) == 1:
+        level = int(level)
+    elif len(parts) == 2:
+        level = float(level)
+
+    return level
